@@ -37,7 +37,6 @@ Copyright (c) Intel Corporation (2009-2017).
 #include "llvm/Support/Casting.h"
 #include "llvm/Support/Path.h"
 #include "llvm/Support/Threading.h"
-#include "llvm/Support/TargetSelect.h"
 #include "llvm/Support/ManagedStatic.h"
 #include "llvm/Support/Mutex.h"
 #include "llvm/Support/VirtualFileSystem.h"
@@ -60,7 +59,6 @@ Copyright (c) Intel Corporation (2009-2017).
 // in https://github.com/KhronosGroup/OpenCL-Headers/blob/master/CL/cl.h
 #define CL_SUCCESS 0
 #define CL_COMPILE_PROGRAM_FAILURE -15
-#define CL_INVALID_BUILD_OPTIONS -43
 #define CL_OUT_OF_HOST_MEMORY -6
 
 #include "assert.h"
@@ -82,9 +80,7 @@ Copyright (c) Intel Corporation (2009-2017).
 
 using namespace Intel::OpenCL::ClangFE;
 
-static volatile bool lazyCCInit =
-    true; // the flag must be 'volatile' to prevent caching in a CPU register
-static llvm::sys::Mutex lazyCCInitMutex;
+llvm::ManagedStatic<llvm::sys::SmartMutex<true>> compileMutex;
 
 void CommonClangTerminate() { llvm::llvm_shutdown(); }
 
@@ -92,23 +88,13 @@ void CommonClangTerminate() { llvm::llvm_shutdown(); }
 // from a DllMain function (Windows specific), or from a function
 // w\ __attribute__ ((constructor)) (Linux specific).
 void CommonClangInitialize() {
-  if (lazyCCInit) {
-    llvm::sys::ScopedLock lock(lazyCCInitMutex);
-
-    if (lazyCCInit) {
-      // CommonClangTerminate calls llvm_shutdown to deallocate resources used
-      // by LLVM libraries. llvm_shutdown uses static mutex to make it safe for
-      // multi-threaded envirounment and LLVM libraries user is expected call
-      // llvm_shutdown before static object are destroyed, so we use atexit to
-      // satisfy this requirement.
-      atexit(CommonClangTerminate);
-      llvm::InitializeAllTargets();
-      llvm::InitializeAllAsmPrinters();
-      llvm::InitializeAllAsmParsers();
-      llvm::InitializeAllTargetMCs();
-      lazyCCInit = false;
-    }
-  }
+  // CommonClangTerminate calls llvm_shutdown to deallocate resources used
+  // by LLVM libraries. llvm_shutdown uses static mutex to make it safe for
+  // multi-threaded envirounment and LLVM libraries user is expected call
+  // llvm_shutdown before static object are destroyed, so we use atexit to
+  // satisfy this requirement.
+  llvm::once_flag OnceFlag;
+  llvm::call_once(OnceFlag, []() { atexit(CommonClangTerminate); });
 }
 
 static bool GetHeaders(std::vector<Resource> &Result) {
@@ -211,6 +197,9 @@ Compile(const char *pszProgramSource, const char **pInputHeaders,
   CommonClangInitialize();
 
   try {
+#ifdef _WIN32
+    llvm::sys::SmartScopedLock<true> compileGuard{*compileMutex};
+#endif
     std::unique_ptr<OCLFEBinaryResult> pResult(new OCLFEBinaryResult());
 
     // Create the clang compiler
@@ -222,12 +211,11 @@ Compile(const char *pszProgramSource, const char **pInputHeaders,
     // Prepare error log
     llvm::raw_string_ostream err_ostream(pResult->getLogRef());
     {
+#ifndef _WIN32
+      llvm::sys::SmartScopedLock<true> compileGuard{*compileMutex};
+#endif
       // Parse options
-      if (optionsParser.processOptions(pszOptions, pszOptionsEx) != 0) {
-        if (pBinaryResult)
-          *pBinaryResult = nullptr;
-        return CL_INVALID_BUILD_OPTIONS;
-      }
+      optionsParser.processOptions(pszOptions, pszOptionsEx);
 
       // Prepare our diagnostic client.
       llvm::IntrusiveRefCntPtr<clang::DiagnosticIDs> DiagID(
@@ -333,10 +321,8 @@ Compile(const char *pszProgramSource, const char **pInputHeaders,
       SmallVectorBuffer StreamBuf(pResult->getIRBufferRef());
       std::ostream OS(&StreamBuf);
       std::string Err;
-      SPIRV::TranslatorOpts SPIRVOpts(SPIRV::VersionNumber::MaximumVersion,
-                                      optionsParser.getSPIRVExtStatusMap());
-      if (!optionsParser.hasSPIRVExt())
-        SPIRVOpts.enableAllExtensions();
+      SPIRV::TranslatorOpts SPIRVOpts;
+      SPIRVOpts.enableAllExtensions();
       if (!optionsParser.hasOptDisable()) {
         SPIRVOpts.setMemToRegEnabled(true);
       }
@@ -346,6 +332,9 @@ Compile(const char *pszProgramSource, const char **pInputHeaders,
       err_ostream.flush();
     }
     {
+#ifndef _WIN32
+      llvm::sys::SmartScopedLock<true> compileGuard{*compileMutex};
+#endif
       if (pBinaryResult) {
         *pBinaryResult = pResult.release();
       }
